@@ -166,12 +166,19 @@ unsigned php_http_array_list(HashTable *ht TSRMLS_DC, unsigned argc, ...)
 	return argl;
 }
 
+void php_http_array_copy_strings(void *zpp)
+{
+	zval **zvpp = ((zval **) zpp);
+
+	*zvpp = php_http_zsep(1, IS_STRING, *zvpp);
+}
+
 int php_http_array_apply_append_func(void *pDest TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
 {
 	int flags;
 	char *key = NULL;
 	HashTable *dst;
-	zval **data = NULL, **value = (zval **) pDest;
+	zval **data = NULL, *value = *((zval **) pDest);
 
 	dst = va_arg(args, HashTable *);
 	flags = va_arg(args, int);
@@ -184,16 +191,21 @@ int php_http_array_apply_append_func(void *pDest TSRMLS_DC, int num_args, va_lis
 			zend_hash_quick_find(dst, hash_key->arKey, hash_key->nKeyLength, hash_key->h, (void *) &data);
 		}
 
-		Z_ADDREF_P(*value);
+		if (flags & ARRAY_JOIN_STRINGIFY) {
+			value = php_http_zsep(1, IS_STRING, value);
+		} else {
+			Z_ADDREF_P(value);
+		}
+
 		if (data) {
 			if (Z_TYPE_PP(data) != IS_ARRAY) {
 				convert_to_array(*data);
 			}
-			add_next_index_zval(*data, *value);
+			add_next_index_zval(*data, value);
 		} else if (key) {
-			zend_symtable_update(dst, key, hash_key->nKeyLength, value, sizeof(zval *), NULL);
+			zend_symtable_update(dst, key, hash_key->nKeyLength, &value, sizeof(zval *), NULL);
 		} else {
-			zend_hash_quick_add(dst, hash_key->arKey, hash_key->nKeyLength, hash_key->h, value, sizeof(zval *), NULL);
+			zend_hash_quick_add(dst, hash_key->arKey, hash_key->nKeyLength, hash_key->h, &value, sizeof(zval *), NULL);
 		}
 
 		if (key) {
@@ -209,19 +221,24 @@ int php_http_array_apply_merge_func(void *pDest TSRMLS_DC, int num_args, va_list
 	int flags;
 	char *key = NULL;
 	HashTable *dst;
-	zval **value = (zval **) pDest;
+	zval *value = *((zval **) pDest);
 
 	dst = va_arg(args, HashTable *);
 	flags = va_arg(args, int);
 
 	if ((!(flags & ARRAY_JOIN_STRONLY)) || hash_key->nKeyLength) {
-		Z_ADDREF_P(*value);
+		if (flags & ARRAY_JOIN_STRINGIFY) {
+			value = php_http_zsep(1, IS_STRING, value);
+		} else {
+			Z_ADDREF_P(value);
+		}
+
 		if ((flags & ARRAY_JOIN_PRETTIFY) && hash_key->nKeyLength) {
 			key = php_http_pretty_key(estrndup(hash_key->arKey, hash_key->nKeyLength - 1), hash_key->nKeyLength - 1, 1, 1);
-			zend_hash_update(dst, key, hash_key->nKeyLength, (void *) value, sizeof(zval *), NULL);
+			zend_hash_update(dst, key, hash_key->nKeyLength, (void *) &value, sizeof(zval *), NULL);
 			efree(key);
 		} else {
-			zend_hash_quick_update(dst, hash_key->arKey, hash_key->nKeyLength, hash_key->h, (void *) value, sizeof(zval *), NULL);
+			zend_hash_quick_update(dst, hash_key->arKey, hash_key->nKeyLength, hash_key->h, (void *) &value, sizeof(zval *), NULL);
 		}
 	}
 
